from tensorflow_technical_indicators.features import open, high, low, close, volume
import tensorflow as tf


class FeaturesTest(tf.test.TestCase):

    def setUp(self):
        super(FeaturesTest, self).setUp()

    def testOHLCVColumnGettersWork(self):

        # Arrange
        candles = tf.constant([
            [0, 1, 2, 3, 4],  # open
            [0, 1, 2, 3, 4],  # high
            [0, 1, 2, 3, 4],  # low
            [0, 1, 2, 3, 4],  # close
            [0, 1, 2, 3, 4]  # volume
        ], dtype=tf.float32)

        # Act
        o = open(candles)
        h = high(candles)
        l = low(candles)
        c = close(candles)
        v = volume(candles)

        # Assert
        self.assertAllClose(o, tf.constant([0, 0, 0, 0, 0], dtype=tf.float32))
        self.assertAllClose(h, tf.constant([1, 1, 1, 1, 1], dtype=tf.float32))
        self.assertAllClose(l, tf.constant([2, 2, 2, 2, 2], dtype=tf.float32))
        self.assertAllClose(c, tf.constant([3, 3, 3, 3, 3], dtype=tf.float32))
        self.assertAllClose(v, tf.constant([4, 4, 4, 4, 4], dtype=tf.float32))


if __name__ == '__main__':
    tf.compat.v1.enable_v2_behavior()
    tf.test.main()



from tensorflow_technical_indicators.features import open, high, low, close, volume
from tensorflow_technical_indicators.moving_average import simple_moving_average, exponential_moving_average

import tensorflow as tf


class MovingAverageTest(tf.test.TestCase):

    def setUp(self):
        super(MovingAverageTest, self).setUp()

    def testSimpleMovingAverageWorksOneDim(self):

        # Arrange
        candles = tf.constant([
            [5],
            [10],
            [15],
            [25]
        ], dtype=tf.float32)

        # Act
        sma = simple_moving_average(
            candles=candles, window_size=2)

        # Assert
        expected = tf.constant([
            [5],
            [7.5],
            [12.5],
            [20]])
        self.assertAllClose(sma, expected)

    def testSimpleMovingAverageWorksManyDims(self):

        # Arrange
        candles = tf.constant([
            [5, 5],
            [10, 15],
            [15, 20],
            [25, 30]
        ], dtype=tf.float32)

        # Act
        sma = simple_moving_average(
            candles=candles, window_size=2)

        # Assert
        expected = tf.constant([
            [5, 5],
            [7.5, 10],
            [12.5, 17.5],
            [20, 25]])
        self.assertAllClose(sma, expected)

    def testExponentialMovingAverageWorksOneDim(self):

        # Arrange
        candles = tf.constant([
            [2.0],
            [4.0],
            [6.0],
            [8.0]
        ], dtype=tf.float32)

        # Act
        ema = exponential_moving_average(
            candles=candles, window_size=2)

        # Assert
        expected = tf.constant([
            [2],
            [3.66666],
            [5.22222],
            [7.074074]])
        self.assertAllClose(ema, expected, rtol=1e-04)

    def testExponentialMovingAverageWorksManyDims(self):

        # Arrange
        candles = tf.constant([
            [2.0, 2.0],
            [4.0, 4.0],
            [6.0, 6.0],
            [8.0, 8.0]
        ], dtype=tf.float32)

        # Act
        ema = exponential_moving_average(
            candles=candles, window_size=2)

        # Assert
        expected = tf.constant([
            [2, 2],
            [3.66666, 3.66666],
            [5.22222, 5.2222],
            [7.074074, 7.074074]])
        self.assertAllClose(ema, expected, rtol=1e-04)


if __name__ == '__main__':
    tf.compat.v1.enable_v2_behavior()
    tf.test.main()



import tensorflow as tf


@tf.function
def _get_col(candles, idx):
    return candles[:, idx]


@tf.function
def open(candles):
    return _get_col(candles, 0)


@tf.function
def high(candles):
    return _get_col(candles, 1)


@tf.function
def low(candles):
    return _get_col(candles, 2)


@tf.function
def close(candles):
    return _get_col(candles, 3)


@tf.function
def volume(candles):
    return _get_col(candles, 4)



import tensorflow as tf
from .moving_average import simple_moving_average, exponential_moving_average


@tf.function
def macd(series, a=12, b=26, c=9):
    ema_a = exponential_moving_average(series, a)
    ema_b = exponential_moving_average(series, b)

    signal_line = exponential_moving_average(series, c)

    macd_line = ema_a - ema_b
    return macd_line - signal_line



import tensorflow as tf
from .moving_average import simple_moving_average, exponential_moving_average


@tf.function
def upday(series):
    padded_series = tf.concat(
        [tf.zeros((1, series.shape[1])), series], axis=0)

    series_prev = padded_series[:-1]
    return tf.maximum(series - series_prev, tf.zeros(series.shape))


@ tf.function
def downday(series):
    padded_series = tf.concat(
        [tf.zeros((1, series.shape[1])), series], axis=0)

    series_prev = padded_series[:-1]
    return tf.maximum(series_prev - series, tf.zeros(series.shape))


@ tf.function
def rsi(series, time_window=7, method='sma'):
    up = upday(series)
    down = downday(series)

    maup = simple_moving_average(
        up, time_window) if method == 'sma' else exponential_moving_average(up, time_window)
    madown = simple_moving_average(
        down, time_window) if method == 'sma' else exponential_moving_average(down, time_window)

    rs = maup / madown

    return 100 - 100 / (1 + rs)



import tensorflow as tf


@tf.function
def simple_moving_average(candles, window_size=7):

    moving_average = tf.TensorArray(tf.float32, candles.shape[0])

    for i in tf.range(start=0, limit=window_size-1):
        moving_average = moving_average.write(i, candles[i])

    for i in tf.range(start=window_size-1, limit=candles.shape[0]):
        window = candles[i-1:i+window_size-1]
        ma = tf.reduce_sum(window, axis=0) / window_size
        moving_average = moving_average.write(i, ma)

    return moving_average.stack()


@tf.function
def exponential_moving_average(candles, window_size=9):
    moving_average = tf.TensorArray(tf.float32, candles.shape[0])

    initial_sma = tf.reduce_sum(candles[:window_size], axis=0) / window_size
    multiplier = 2 / (window_size + 1)

    moving_average = moving_average.write(window_size-2, initial_sma)

    for i in tf.range(start=window_size-1, limit=candles.shape[0]):
        prev_ema = moving_average.read(i-1)

        next_ema = (candles[i] * multiplier) + (prev_ema * (1 - multiplier))

        moving_average = moving_average.write(i, next_ema)

    for i in tf.range(start=0, limit=window_size-1):
        moving_average = moving_average.write(i, candles[i])

    return moving_average.stack()



from tensorflow_technical_indicators.macd import macd

import tensorflow as tf


class RsiTest(tf.test.TestCase):

    def setUp(self):
        super(RsiTest, self).setUp()

    def testMacdWorks(self):
        # Arrange
        series = tf.constant(
            [[1, 1], [2, 1], [1, 1], [4, 1], [5, 1], [5, 1], [9, 1]], dtype=tf.float32)

        # Act
        result = macd(series, a=2, b=4, c=2)

        # Assert
        expected = tf.constant(
            [[-1, -1], [-2, -1], [-1, -1], [-2.8, -1], [-3.68, -1], [-4.208, -1], [-6.124, -1]])
        self.assertAllClose(result, expected, rtol=1e-03)


if __name__ == '__main__':
    tf.compat.v1.enable_v2_behavior()
    tf.test.main()



from tensorflow_technical_indicators.rsi import rsi, upday, downday

import tensorflow as tf


class RsiTest(tf.test.TestCase):

    def setUp(self):
        super(RsiTest, self).setUp()

    def testUpDayWorks(self):
        # Arrange
        series = tf.constant(
            [[1, 1], [2, 2], [1, 1], [5, 4]], dtype=tf.float32)

        # Act
        upd = upday(series)

        # Assert
        expected = tf.constant([[1, 1], [1, 1], [0, 0], [4, 3]])
        self.assertAllClose(upd, expected)

    def testDownDayWorks(self):
        # Arrange
        series = tf.constant(
            [[1, 3], [2, 2], [1, 1], [5, 5]], dtype=tf.float32)

        # Act
        dd = downday(series)

        # Assert
        expected = tf.constant([[0, 0], [0, 1], [1, 1], [0, 0]])
        self.assertAllClose(dd, expected)

    def testRsiWorks(self):
        # Arrange
        series = tf.constant(
            [[1], [2], [1], [5], [3], [6], [2]], dtype=tf.float32)

        # Act
        result = rsi(series, time_window=2)

        # Assert
        expected = tf.constant(
            [[100], [100], [50], [80], [66.6666], [60], [42.857143]])
        self.assertAllClose(result, expected, rtol=1e-02)

    def testRsiExponentialDoesNotCrash(self):
        # Arrange
        series = tf.constant(
            [[1], [2], [1], [5], [3], [6], [2]], dtype=tf.float32)

        # Act
        rsi(series, time_window=2, method='ema')


if __name__ == '__main__':
    tf.compat.v1.enable_v2_behavior()
    tf.test.main()
